#include <stdio.h>
#include <limits.h> /* CHAR_BIT */

/* Count number of 1 bits ("population count") */
long pcount_do(unsigned long x)
{
	long result = 0;
	do {
		/* Add 1 if lowest bit is set */
		result += x & 0x1;
		/* Shift off the lowest bit */
		x >>= 1;
	} while (x);

	return result;
}

long pcount_goto(unsigned long x)
{
	long result = 0;
loop:
	result += x & 0x1;
	x >>= 1;
	if (x) goto loop;

	return result;
}

long pcount_while(unsigned long x)
{
	long result = 0;
	while (x) {
		result += x & 0x1;
		x >>= 1;
	}

	return result;
}

long pcount_goto_jtm(unsigned long x)
{
	long result = 0;
	goto test;
loop:
	result += x & 0x1;
	x >>= 1;
test:
	if (x) goto loop;

	return result;
}

long pcount_goto_ifdo(unsigned long x)
{
	long result = 0;
	if (!x) goto done;
loop:
	result += x & 0x1;
	x >>= 1;
	if (x) goto loop;
done:

	return result;
}

#define WSIZE (CHAR_BIT * sizeof(int))
long pcount_for(unsigned long x)
{
	size_t i;
	long result = 0;
	for (i = 0; i < WSIZE; i++)
	{
		unsigned bit = (x >> i) & 0x1;
		result += bit;
	}
	return result;
}

long pcount_for_while(unsigned long x)
{
	size_t i;
	long result = 0;
	i = 0;
	while (i < WSIZE)
	{
		unsigned bit = (x >> i) & 0x1;
		result += bit;
		i++;
	}
	return result;
}

long pcount_goto_for(unsigned long x)
{
	size_t i;
	long result = 0;

	i = 0; /* initialization */
	/* Pre-test: in this case can be optimized away! */
	if (!(i < WSIZE)) goto done;
loop:
	{ /* body */
		unsigned bit = (x >> i) & 0x1;
		result += bit;
	}
	i++;  /* update */
	if (i < WSIZE) goto loop; /* test */
done:
	return result;
}

int main(void)
{
	unsigned long x = pcount_do(0xc5485006);
	unsigned long y = pcount_goto(0xc5485006);
	printf("pop count = %lu, %lu\n", x, y);
	return 0;
}
